/**
 * Main Application script
 */
import "./App.scss";

import React, { FunctionComponent, useEffect, useRef } from "react";
import { BryntumSchedulerPro } from "@bryntum/schedulerpro-react";
import _ from "lodash";

import { useDispatch, useSelector } from "react-redux";

import { schedulerproConfig } from "./SchedulerProConfig";
import store, { Dispatch } from "./store";

const App: FunctionComponent = () => {
  const schedulerpro = useRef<BryntumSchedulerPro>(null);

  const events = useSelector(store.select.dataModel.events);
  const eventAssignments = useSelector(store.select.dataModel.eventAssignments);
  const resources = useSelector(store.select.dataModel.resources);

  const dispatch = useDispatch<Dispatch>();

  useEffect(() => {
    dispatch.dataModel.init();
  }, []);

  /* 
  BUG APPEARS WHEN:: REASSIGN THE DATA ON CHANGE TO FREDY
  */
  const handleDataChange = (source: any) => {
    console.log("xxx", source);
    dispatch.dataModel.moveWorkToFredy({
      eventId: source.eventRecord.id,
    });

    source.valid = true;
  };

  const clonedEvents = _.cloneDeep([...events]) || [];
  console.log("clonedEvents", clonedEvents);
  const clonedEventAssignments = _.cloneDeep([...eventAssignments]) || [];
  console.log("clonedEventAssignments", clonedEventAssignments);
  const clonedResources = _.cloneDeep([...resources]) || [];
  console.log("clonedResources", clonedResources);

  const tweakedEvents = clonedEvents.map((e) => ({
    ...e,
    duration: undefined,
  }));

  return (
    <>
      <button onClick={() => dispatch.dataModel.init()}>refresh</button>
      {clonedResources ? (
        <BryntumSchedulerPro
          ref={schedulerpro}
          resources={clonedResources}
          assignments={clonedResources.length > 0 ? clonedEventAssignments : []}
          events={clonedResources.length > 0 ? tweakedEvents : []}
          onEventResizeEnd={handleDataChange}
          {...schedulerproConfig}
        />
      ) : null}
    </>
  );
};

// If you plan to use stateful React collections for data binding please check this guide
// https://bryntum.com/products/schedulerpro/docs/guide/SchedulerPro/integration/react/data-binding

export default App;
