/**
 * Application configuration
 */
import { AssignmentStore, EventStore } from "@bryntum/schedulerpro";
import { BryntumSchedulerProProps } from "@bryntum/schedulerpro-react";

class CustomizedEventStore extends EventStore {
  static get defaultConfig() {
    return {
      singleAssignment: false,
      autoLoad: true,
      autoCommit: true,
    };
  }
}

const schedulerproConfig: Partial<BryntumSchedulerProProps> = {
  startDate: "2023-02-01 5:00",
  endDate: "2023-02-01 22:00",
  rowHeight: 60,
  barMargin: 15,
  eventStyle: "colored",
  viewPreset: "hourAndDay",

  columns: [
    {
      field: "id",
      text: "name",
      width: 400,
      showEventCount: false, // count of events per resource, if disabled then showMeta is used
      enableCellContextMenu: false, // disable delete/move options for resources
      editor: false, // disable edit of resource name
      showRole: false,
    },
  ],

  createEventOnDblClick: false, // prevent automatic event creation
  enableDeleteKey: false,
  multiEventSelect: true, // select multiple entries within the gantt chart and move them / dispatch them
  allowOverlap: true, // allow overlapping events

  eventDragSelectFeature: {
    // prevent automatic event creation
    disabled: true,
  },
  eventDragCreateFeature: {
    disabled: true,
  },
  eventCopyPasteFeature: {
    disabled: true,
  },
  eventDragFeature: {
    constrainDragToTimeline: false,
    showTooltip: false, // show tooltip with dates on top of event
  },

  eventStore: new CustomizedEventStore(),
  assignmentStore: new AssignmentStore({
    autoLoad: true,
    autoCommit: true,
  }),
};

export { schedulerproConfig };
