import { createModel } from "@rematch/core";

import { RootModel } from "./index";

import data from "./data.json";

export interface DataModelState {
  events: Map<string, any>;
  eventAssignments: Map<string, any>;
  resources: Map<string, any>;
}

type Setter = {
  key: keyof DataModelState;
  value: any;
};

type MoveWorkToFredy = {
  eventId: string;
};

const initialState: DataModelState = {
  events: new Map(),
  eventAssignments: new Map(),
  resources: new Map(),
};

export const dataModel = createModel<RootModel>()({
  state: initialState,
  reducers: {
    setter(state, payload: Setter) {
      const { key, value } = payload;

      state[key] = value;
    },
    moveWorkToFredy(state, payload: MoveWorkToFredy) {
      const { eventId } = payload;

      const event = state.events.get(eventId);

      if (event) {
        state.events.set(eventId, {
          ...event,
          randomData: "updated",
        });
      }

      state.eventAssignments.forEach((assignment) => {
        if (assignment.eventId === eventId) {
          state.eventAssignments.set(assignment.id, {
            ...assignment,
            resourceId: "Fredy",
          });
        }
      });
    },
  },
  effects: (dispatch) => ({
    init() {
      // EVENT
      const eventMap = new Map();
      for (const item of data.events) {
        eventMap.set(item.id, item);
      }
      dispatch.dataModel.setter({
        key: "events",
        value: eventMap,
      });

      // ASSIGNMENT
      const assginmentsMap = new Map();
      for (const item of data.eventAssignments) {
        assginmentsMap.set(item.id, item);
      }
      dispatch.dataModel.setter({
        key: "eventAssignments",
        value: assginmentsMap,
      });

      const resourceMap = new Map();
      for (const item of data.resources) {
        resourceMap.set(item.id, item);
      }
      dispatch.dataModel.setter({
        key: "resources",
        value: resourceMap,
      });
    },
  }),
  selectors: (slice) => ({
    events: () =>
      slice((state) => Array.from(state.events, ([, value]) => value)),
    eventAssignments: () =>
      slice((state) =>
        Array.from(state.eventAssignments, ([, value]) => value)
      ),
    resources: () =>
      slice((state) => Array.from(state.resources, ([, value]) => value)),
  }),
});
